# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogCustomAttributeDefinitionStringConfig(UncheckedBaseModel):
    """
    Configuration associated with Custom Attribute Definitions of type `STRING`.
    """

    enforce_uniqueness: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If true, each Custom Attribute instance associated with this Custom Attribute
    Definition must have a unique value within the seller's catalog. For
    example, this may be used for a value like a SKU that should not be
    duplicated within a seller's catalog. May not be modified after the
    definition has been created.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class SelectOptionParams(typing_extensions.TypedDict):
    reference_id: str
    """
    The reference id for the option.
    """

    title: str
    """
    The title text that displays in the select option button.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class DateRange(UncheckedBaseModel):
    """
    A range defined by two dates. Used for filtering a query for Connect v2
    objects that have date properties.
    """

    start_date: typing.Optional[str] = pydantic.Field(default=None)
    """
    A string in `YYYY-MM-DD` format, such as `2017-10-31`, per the ISO 8601
    extended format for calendar dates.
    The beginning of a date range (inclusive).
    """

    end_date: typing.Optional[str] = pydantic.Field(default=None)
    """
    A string in `YYYY-MM-DD` format, such as `2017-10-31`, per the ISO 8601
    extended format for calendar dates.
    The end of a date range (inclusive).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .checkout_location_settings_policy import CheckoutLocationSettingsPolicyParams
from .checkout_location_settings_branding import CheckoutLocationSettingsBrandingParams
from .checkout_location_settings_tipping import CheckoutLocationSettingsTippingParams
from .checkout_location_settings_coupons import CheckoutLocationSettingsCouponsParams


class CheckoutLocationSettingsParams(typing_extensions.TypedDict):
    location_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the location that these settings apply to.
    """

    customer_notes_enabled: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether customers are allowed to leave notes at checkout.
    """

    policies: typing_extensions.NotRequired[typing.Optional[typing.Sequence[CheckoutLocationSettingsPolicyParams]]]
    """
    Policy information is displayed at the bottom of the checkout pages.
    You can set a maximum of two policies.
    """

    branding: typing_extensions.NotRequired[CheckoutLocationSettingsBrandingParams]
    """
    The branding settings for this location.
    """

    tipping: typing_extensions.NotRequired[CheckoutLocationSettingsTippingParams]
    """
    The tip settings for this location.
    """

    coupons: typing_extensions.NotRequired[CheckoutLocationSettingsCouponsParams]
    """
    The coupon settings for this location.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the settings were last updated, in RFC 3339 format.
    Examples for January 25th, 2020 6:25:34pm Pacific Standard Time:
    UTC: 2020-01-26T02:25:34Z
    Pacific Standard Time with UTC offset: 2020-01-25T18:25:34-08:00
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class ScheduledShiftDetailsParams(typing_extensions.TypedDict):
    """
    Represents shift details for draft and published versions of a [scheduled shift](entity:ScheduledShift),
    such as job ID, team member assignment, and start and end times.
    """

    team_member_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the [team member](entity:TeamMember) scheduled for the shift.
    """

    location_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the [location](entity:Location) the shift is scheduled for.
    """

    job_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the [job](entity:Job) the shift is scheduled for.
    """

    start_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The start time of the shift, in RFC 3339 format in the time zone &plus;
    offset of the shift location specified in `location_id`. Precision up to the minute
    is respected; seconds are truncated.
    """

    end_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The end time for the shift, in RFC 3339 format in the time zone &plus;
    offset of the shift location specified in `location_id`. Precision up to the minute
    is respected; seconds are truncated.
    """

    notes: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Optional notes for the shift.
    """

    is_deleted: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether the draft shift version is deleted. If set to `true` when the shift
    is published, the entire scheduled shift (including the published shift) is deleted and
    cannot be accessed using any endpoint.
    """

    timezone: typing_extensions.NotRequired[str]
    """
    The time zone of the shift location, calculated based on the `location_id`. This field
    is provided for convenience.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .customer import CustomerParams


class GetCustomerResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields that are included in the response body of
    a request to the `RetrieveCustomer` endpoint.

    Either `errors` or `customer` is present in a given response (never both).
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    customer: typing_extensions.NotRequired[CustomerParams]
    """
    The requested customer.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.inventory_state import InventoryState
from .source_application import SourceApplicationParams


class InventoryTransferParams(typing_extensions.TypedDict):
    """
    Represents the transfer of a quantity of product inventory at a
    particular time from one location to another.
    """

    id: typing_extensions.NotRequired[str]
    """
    A unique ID generated by Square for the
    `InventoryTransfer`.
    """

    reference_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    An optional ID provided by the application to tie the
    `InventoryTransfer` to an external system.
    """

    state: typing_extensions.NotRequired[InventoryState]
    """
    The [inventory state](entity:InventoryState) for the quantity of
    items being transferred.
    See [InventoryState](#type-inventorystate) for possible values
    """

    from_location_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The Square-generated ID of the [Location](entity:Location) where the related
    quantity of items was tracked before the transfer.
    """

    to_location_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The Square-generated ID of the [Location](entity:Location) where the related
    quantity of items was tracked after the transfer.
    """

    catalog_object_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The Square-generated ID of the
    [CatalogObject](entity:CatalogObject) being tracked.
    """

    catalog_object_type: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The [type](entity:CatalogObjectType) of the [CatalogObject](entity:CatalogObject) being tracked. 
    
    The Inventory API supports setting and reading the `"catalog_object_type": "ITEM_VARIATION"` field value. 
    In addition, it can also read the `"catalog_object_type": "ITEM"` field value that is set by the Square Restaurants app.
    """

    quantity: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The number of items affected by the transfer as a decimal string.
    Can support up to 5 digits after the decimal point.
    """

    occurred_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A client-generated RFC 3339-formatted timestamp that indicates when
    the transfer took place. For write actions, the `occurred_at` timestamp
    cannot be older than 24 hours or in the future relative to the time of the
    request.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    An RFC 3339-formatted timestamp that indicates when Square
    received the transfer request.
    """

    source: typing_extensions.NotRequired[SourceApplicationParams]
    """
    Information about the application that initiated the
    inventory transfer.
    """

    employee_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The Square-generated ID of the [Employee](entity:Employee) responsible for the
    inventory transfer.
    """

    team_member_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The Square-generated ID of the [Team Member](entity:TeamMember) responsible for the
    inventory transfer.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class CustomFieldParams(typing_extensions.TypedDict):
    """
    Describes a custom form field to add to the checkout page to collect more information from buyers during checkout.
    For more information,
    see [Specify checkout options](https://developer.squareup.com/docs/checkout-api/optional-checkout-configurations#specify-checkout-options-1).
    """

    title: str
    """
    The title of the custom field.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.v1order_history_entry_action import V1OrderHistoryEntryAction


class V1OrderHistoryEntryParams(typing_extensions.TypedDict):
    """
    V1OrderHistoryEntry
    """

    action: typing_extensions.NotRequired[V1OrderHistoryEntryAction]
    """
    The type of action performed on the order.
    See [V1OrderHistoryEntryAction](#type-v1orderhistoryentryaction) for possible values
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The time when the action was performed, in ISO 8601 format.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .dispute import DisputeParams


class ListDisputesResponseParams(typing_extensions.TypedDict):
    """
    Defines fields in a `ListDisputes` response.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information about errors encountered during the request.
    """

    disputes: typing_extensions.NotRequired[typing.Sequence[DisputeParams]]
    """
    The list of disputes.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    The pagination cursor to be used in a subsequent request.
    If unset, this is the final response. For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """



# This file was auto-generated by Fern from our API Definition.

import typing

TenderCardDetailsEntryMethod = typing.Union[
    typing.Literal["SWIPED", "KEYED", "EMV", "ON_FILE", "CONTACTLESS"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .loyalty_event_loyalty_account_filter import LoyaltyEventLoyaltyAccountFilterParams
from .loyalty_event_type_filter import LoyaltyEventTypeFilterParams
from .loyalty_event_date_time_filter import LoyaltyEventDateTimeFilterParams
from .loyalty_event_location_filter import LoyaltyEventLocationFilterParams
from .loyalty_event_order_filter import LoyaltyEventOrderFilterParams


class LoyaltyEventFilterParams(typing_extensions.TypedDict):
    """
    The filtering criteria. If the request specifies multiple filters,
    the endpoint uses a logical AND to evaluate them.
    """

    loyalty_account_filter: typing_extensions.NotRequired[LoyaltyEventLoyaltyAccountFilterParams]
    """
    Filter events by loyalty account.
    """

    type_filter: typing_extensions.NotRequired[LoyaltyEventTypeFilterParams]
    """
    Filter events by event type.
    """

    date_time_filter: typing_extensions.NotRequired[LoyaltyEventDateTimeFilterParams]
    """
    Filter events by date time range. 
    For each range, the start time is inclusive and the end time 
    is exclusive.
    """

    location_filter: typing_extensions.NotRequired[LoyaltyEventLocationFilterParams]
    """
    Filter events by location.
    """

    order_filter: typing_extensions.NotRequired[LoyaltyEventOrderFilterParams]
    """
    Filter events by the order associated with the event.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..types.sort_order import SortOrder
from ..core.request_options import RequestOptions
from ..core.http_response import HttpResponse
from ..types.v1order import V1Order
from ..core.jsonable_encoder import jsonable_encoder
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.v1update_order_request_action import V1UpdateOrderRequestAction
from ..core.client_wrapper import AsyncClientWrapper
from ..core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawV1TransactionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def v1list_orders(
        self,
        location_id: str,
        *,
        order: typing.Optional[SortOrder] = None,
        limit: typing.Optional[int] = None,
        batch_token: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.List[V1Order]]:
        """
        Provides summary information for a merchant's online store orders.

        Parameters
        ----------
        location_id : str
            The ID of the location to list online store orders for.

        order : typing.Optional[SortOrder]
            The order in which payments are listed in the response.

        limit : typing.Optional[int]
            The maximum number of payments to return in a single response. This value cannot exceed 200.

        batch_token : typing.Optional[str]
            A pagination cursor to retrieve the next set of results for your
            original query to the endpoint.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[V1Order]]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/{jsonable_encoder(location_id)}/orders",
            method="GET",
            params={
                "order": order,
                "limit": limit,
                "batch_token": batch_token,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[V1Order],
                    construct_type(
                        type_=typing.List[V1Order],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def v1retrieve_order(
        self, location_id: str, order_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[V1Order]:
        """
        Provides comprehensive information for a single online store order, including the order's history.

        Parameters
        ----------
        location_id : str
            The ID of the order's associated location.

        order_id : str
            The order's Square-issued ID. You obtain this value from Order objects returned by the List Orders endpoint

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[V1Order]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/{jsonable_encoder(location_id)}/orders/{jsonable_encoder(order_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    V1Order,
                    construct_type(
                        type_=V1Order,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def v1update_order(
        self,
        location_id: str,
        order_id: str,
        *,
        action: V1UpdateOrderRequestAction,
        shipped_tracking_number: typing.Optional[str] = OMIT,
        completed_note: typing.Optional[str] = OMIT,
        refunded_note: typing.Optional[str] = OMIT,
        canceled_note: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[V1Order]:
        """
        Updates the details of an online store order. Every update you perform on an order corresponds to one of three actions:

        Parameters
        ----------
        location_id : str
            The ID of the order's associated location.

        order_id : str
            The order's Square-issued ID. You obtain this value from Order objects returned by the List Orders endpoint

        action : V1UpdateOrderRequestAction
            The action to perform on the order (COMPLETE, CANCEL, or REFUND).
            See [V1UpdateOrderRequestAction](#type-v1updateorderrequestaction) for possible values

        shipped_tracking_number : typing.Optional[str]
            The tracking number of the shipment associated with the order. Only valid if action is COMPLETE.

        completed_note : typing.Optional[str]
            A merchant-specified note about the completion of the order. Only valid if action is COMPLETE.

        refunded_note : typing.Optional[str]
            A merchant-specified note about the refunding of the order. Only valid if action is REFUND.

        canceled_note : typing.Optional[str]
            A merchant-specified note about the canceling of the order. Only valid if action is CANCEL.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[V1Order]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/{jsonable_encoder(location_id)}/orders/{jsonable_encoder(order_id)}",
            method="PUT",
            json={
                "action": action,
                "shipped_tracking_number": shipped_tracking_number,
                "completed_note": completed_note,
                "refunded_note": refunded_note,
                "canceled_note": canceled_note,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    V1Order,
                    construct_type(
                        type_=V1Order,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawV1TransactionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def v1list_orders(
        self,
        location_id: str,
        *,
        order: typing.Optional[SortOrder] = None,
        limit: typing.Optional[int] = None,
        batch_token: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.List[V1Order]]:
        """
        Provides summary information for a merchant's online store orders.

        Parameters
        ----------
        location_id : str
            The ID of the location to list online store orders for.

        order : typing.Optional[SortOrder]
            The order in which payments are listed in the response.

        limit : typing.Optional[int]
            The maximum number of payments to return in a single response. This value cannot exceed 200.

        batch_token : typing.Optional[str]
            A pagination cursor to retrieve the next set of results for your
            original query to the endpoint.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[V1Order]]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/{jsonable_encoder(location_id)}/orders",
            method="GET",
            params={
                "order": order,
                "limit": limit,
                "batch_token": batch_token,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[V1Order],
                    construct_type(
                        type_=typing.List[V1Order],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def v1retrieve_order(
        self, location_id: str, order_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[V1Order]:
        """
        Provides comprehensive information for a single online store order, including the order's history.

        Parameters
        ----------
        location_id : str
            The ID of the order's associated location.

        order_id : str
            The order's Square-issued ID. You obtain this value from Order objects returned by the List Orders endpoint

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[V1Order]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/{jsonable_encoder(location_id)}/orders/{jsonable_encoder(order_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    V1Order,
                    construct_type(
                        type_=V1Order,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def v1update_order(
        self,
        location_id: str,
        order_id: str,
        *,
        action: V1UpdateOrderRequestAction,
        shipped_tracking_number: typing.Optional[str] = OMIT,
        completed_note: typing.Optional[str] = OMIT,
        refunded_note: typing.Optional[str] = OMIT,
        canceled_note: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[V1Order]:
        """
        Updates the details of an online store order. Every update you perform on an order corresponds to one of three actions:

        Parameters
        ----------
        location_id : str
            The ID of the order's associated location.

        order_id : str
            The order's Square-issued ID. You obtain this value from Order objects returned by the List Orders endpoint

        action : V1UpdateOrderRequestAction
            The action to perform on the order (COMPLETE, CANCEL, or REFUND).
            See [V1UpdateOrderRequestAction](#type-v1updateorderrequestaction) for possible values

        shipped_tracking_number : typing.Optional[str]
            The tracking number of the shipment associated with the order. Only valid if action is COMPLETE.

        completed_note : typing.Optional[str]
            A merchant-specified note about the completion of the order. Only valid if action is COMPLETE.

        refunded_note : typing.Optional[str]
            A merchant-specified note about the refunding of the order. Only valid if action is REFUND.

        canceled_note : typing.Optional[str]
            A merchant-specified note about the canceling of the order. Only valid if action is CANCEL.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[V1Order]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/{jsonable_encoder(location_id)}/orders/{jsonable_encoder(order_id)}",
            method="PUT",
            json={
                "action": action,
                "shipped_tracking_number": shipped_tracking_number,
                "completed_note": completed_note,
                "refunded_note": refunded_note,
                "canceled_note": canceled_note,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    V1Order,
                    construct_type(
                        type_=V1Order,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .cash_drawer_shift_summary import CashDrawerShiftSummaryParams


class ListCashDrawerShiftsResponseParams(typing_extensions.TypedDict):
    cursor: typing_extensions.NotRequired[str]
    """
    Opaque cursor for fetching the next page of results. Cursor is not
    present in the last page of results.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    cash_drawer_shifts: typing_extensions.NotRequired[typing.Sequence[CashDrawerShiftSummaryParams]]
    """
    A collection of CashDrawerShiftSummary objects for shifts that match
    the query.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CardPaymentTimeline(UncheckedBaseModel):
    """
    The timeline for card payments.
    """

    authorized_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the payment was authorized, in RFC 3339 format.
    """

    captured_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the payment was captured, in RFC 3339 format.
    """

    voided_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the payment was voided, in RFC 3339 format.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class OrderReward(UncheckedBaseModel):
    """
    Represents a reward that can be applied to an order if the necessary
    reward tier criteria are met. Rewards are created through the Loyalty API.
    """

    id: str = pydantic.Field()
    """
    The identifier of the reward.
    """

    reward_tier_id: str = pydantic.Field()
    """
    The identifier of the reward tier corresponding to this reward.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .time_range import TimeRangeParams


class LoyaltyEventDateTimeFilterParams(typing_extensions.TypedDict):
    """
    Filter events by date time range.
    """

    created_at: TimeRangeParams
    """
    The `created_at` date time range used to filter the result.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class CatalogQueryExact(UncheckedBaseModel):
    """
    The query filter to return the search result by exact match of the specified attribute name and value.
    """

    attribute_name: str = pydantic.Field()
    """
    The name of the attribute to be searched. Matching of the attribute name is exact.
    """

    attribute_value: str = pydantic.Field()
    """
    The desired value of the search attribute. Matching of the attribute value is case insensitive and can be partial.
    For example, if a specified value of "sma", objects with the named attribute value of "Small", "small" are both matched.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams


class BatchDeleteCatalogObjectsResponseParams(typing_extensions.TypedDict):
    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    deleted_object_ids: typing_extensions.NotRequired[typing.Sequence[str]]
    """
    The IDs of all CatalogObjects deleted by this request.
    """

    deleted_at: typing_extensions.NotRequired[str]
    """
    The database [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates) of this deletion in RFC 3339 format, e.g., "2016-09-04T23:59:33.123Z".
    """



# This file was auto-generated by Fern from our API Definition.

import typing

CatalogCustomAttributeDefinitionType = typing.Union[
    typing.Literal["STRING", "BOOLEAN", "NUMBER", "SELECTION"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .event_type_metadata import EventTypeMetadataParams


class ListEventTypesResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields that are included in the response body of
    a request to the [ListEventTypes](api-endpoint:Events-ListEventTypes) endpoint.

    Note: if there are errors processing the request, the event types field will not be
    present.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information on errors encountered during the request.
    """

    event_types: typing_extensions.NotRequired[typing.Sequence[str]]
    """
    The list of event types.
    """

    metadata: typing_extensions.NotRequired[typing.Sequence[EventTypeMetadataParams]]
    """
    Contains the metadata of an event type. For more information, see [EventTypeMetadata](entity:EventTypeMetadata).
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .inventory_count import InventoryCountParams


class BatchGetInventoryCountsResponseParams(typing_extensions.TypedDict):
    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    counts: typing_extensions.NotRequired[typing.Sequence[InventoryCountParams]]
    """
    The current calculated inventory counts for the requested objects
    and locations.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    The pagination cursor to be used in a subsequent request. If unset,
    this is the final response.
    
    See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.
    """



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .catalog_category_type import CatalogCategoryType
from .catalog_ecom_seo_data import CatalogEcomSeoData
from .category_path_to_root_node import CategoryPathToRootNode
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.pydantic_utilities import update_forward_refs


class CatalogCategory(UncheckedBaseModel):
    """
    A category to which a `CatalogItem` instance belongs.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The category name. This is a searchable attribute for use in applicable query filters, and its value length is of Unicode code points.
    """

    image_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The IDs of images associated with this `CatalogCategory` instance.
    Currently these images are not displayed by Square, but are free to be displayed in 3rd party applications.
    """

    category_type: typing.Optional[CatalogCategoryType] = pydantic.Field(default=None)
    """
    The type of the category.
    See [CatalogCategoryType](#type-catalogcategorytype) for possible values
    """

    parent_category: typing.Optional["CatalogObjectCategory"] = pydantic.Field(default=None)
    """
    The ID of the parent category of this category instance.
    """

    is_top_level: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether a category is a top level category, which does not have any parent_category.
    """

    channels: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of IDs representing channels, such as a Square Online site, where the category can be made visible.
    """

    availability_period_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The IDs of the `CatalogAvailabilityPeriod` objects associated with the category.
    """

    online_visibility: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether the category is visible (`true`) or hidden (`false`) on all of the seller's Square Online sites.
    """

    root_category: typing.Optional[str] = pydantic.Field(default=None)
    """
    The top-level category in a category hierarchy.
    """

    ecom_seo_data: typing.Optional[CatalogEcomSeoData] = pydantic.Field(default=None)
    """
    The SEO data for a seller's Square Online store.
    """

    path_to_root: typing.Optional[typing.List[CategoryPathToRootNode]] = pydantic.Field(default=None)
    """
    The path from the category to its root category. The first node of the path is the parent of the category
    and the last is the root category. The path is empty if the category is a root category.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .catalog_object_category import CatalogObjectCategory  # noqa: E402

update_forward_refs(CatalogCategory)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .create_vendor_response import CreateVendorResponseParams


class BatchCreateVendorsResponseParams(typing_extensions.TypedDict):
    """
    Represents an output from a call to [BulkCreateVendors](api-endpoint:Vendors-BulkCreateVendors).
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    responses: typing_extensions.NotRequired[typing.Dict[str, CreateVendorResponseParams]]
    """
    A set of [CreateVendorResponse](entity:CreateVendorResponse) objects encapsulating successfully created [Vendor](entity:Vendor)
    objects or error responses for failed attempts. The set is represented by 
    a collection of idempotency-key/`Vendor`-object or idempotency-key/error-object pairs. The idempotency keys correspond to those specified
    in the input.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .search_orders_state_filter import SearchOrdersStateFilterParams
from .search_orders_date_time_filter import SearchOrdersDateTimeFilterParams
from .search_orders_fulfillment_filter import SearchOrdersFulfillmentFilterParams
from .search_orders_source_filter import SearchOrdersSourceFilterParams
from .search_orders_customer_filter import SearchOrdersCustomerFilterParams


class SearchOrdersFilterParams(typing_extensions.TypedDict):
    """
    Filtering criteria to use for a `SearchOrders` request. Multiple filters
    are ANDed together.
    """

    state_filter: typing_extensions.NotRequired[SearchOrdersStateFilterParams]
    """
    Filter by [OrderState](entity:OrderState).
    """

    date_time_filter: typing_extensions.NotRequired[SearchOrdersDateTimeFilterParams]
    """
    Filter for results within a time range.
    
    __Important:__ If you filter for orders by time range, you must set `SearchOrdersSort`
    to sort by the same field.
    [Learn more about filtering orders by time range.](https://developer.squareup.com/docs/orders-api/manage-orders/search-orders#important-note-about-filtering-orders-by-time-range)
    """

    fulfillment_filter: typing_extensions.NotRequired[SearchOrdersFulfillmentFilterParams]
    """
    Filter by the fulfillment type or state.
    """

    source_filter: typing_extensions.NotRequired[SearchOrdersSourceFilterParams]
    """
    Filter by the source of the order.
    """

    customer_filter: typing_extensions.NotRequired[SearchOrdersCustomerFilterParams]
    """
    Filter by customers associated with the order.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawActionsClient
from ...requests.terminal_action import TerminalActionParams
from ...core.request_options import RequestOptions
from ...types.create_terminal_action_response import CreateTerminalActionResponse
from ...requests.terminal_action_query import TerminalActionQueryParams
from ...types.search_terminal_actions_response import SearchTerminalActionsResponse
from ...types.get_terminal_action_response import GetTerminalActionResponse
from ...types.cancel_terminal_action_response import CancelTerminalActionResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawActionsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ActionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawActionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawActionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawActionsClient
        """
        return self._raw_client

    def create(
        self,
        *,
        idempotency_key: str,
        action: TerminalActionParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateTerminalActionResponse:
        """
        Creates a Terminal action request and sends it to the specified device.

        Parameters
        ----------
        idempotency_key : str
            A unique string that identifies this `CreateAction` request. Keys can be any valid string
            but must be unique for every `CreateAction` request.

            See [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more
            information.

        action : TerminalActionParams
            The Action to create.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateTerminalActionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.terminal.actions.create(
            idempotency_key="thahn-70e75c10-47f7-4ab6-88cc-aaa4076d065e",
            action={
                "device_id": "{{DEVICE_ID}}",
                "deadline_duration": "PT5M",
                "type": "SAVE_CARD",
                "save_card_options": {
                    "customer_id": "{{CUSTOMER_ID}}",
                    "reference_id": "user-id-1",
                },
            },
        )
        """
        response = self._raw_client.create(
            idempotency_key=idempotency_key, action=action, request_options=request_options
        )
        return response.data

    def search(
        self,
        *,
        query: typing.Optional[TerminalActionQueryParams] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchTerminalActionsResponse:
        """
        Retrieves a filtered list of Terminal action requests created by the account making the request. Terminal action requests are available for 30 days.

        Parameters
        ----------
        query : typing.Optional[TerminalActionQueryParams]
            Queries terminal actions based on given conditions and sort order.
            Leaving this unset will return all actions with the default sort order.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for the original query.
            See [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination) for more
            information.

        limit : typing.Optional[int]
            Limit the number of results returned for a single request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchTerminalActionsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.terminal.actions.search(
            query={
                "filter": {"created_at": {"start_at": "2022-04-01T00:00:00.000Z"}},
                "sort": {"sort_order": "DESC"},
            },
            limit=2,
        )
        """
        response = self._raw_client.search(query=query, cursor=cursor, limit=limit, request_options=request_options)
        return response.data

    def get(
        self, action_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetTerminalActionResponse:
        """
        Retrieves a Terminal action request by `action_id`. Terminal action requests are available for 30 days.

        Parameters
        ----------
        action_id : str
            Unique ID for the desired `TerminalAction`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetTerminalActionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.terminal.actions.get(
            action_id="action_id",
        )
        """
        response = self._raw_client.get(action_id, request_options=request_options)
        return response.data

    def cancel(
        self, action_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CancelTerminalActionResponse:
        """
        Cancels a Terminal action request if the status of the request permits it.

        Parameters
        ----------
        action_id : str
            Unique ID for the desired `TerminalAction`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CancelTerminalActionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.terminal.actions.cancel(
            action_id="action_id",
        )
        """
        response = self._raw_client.cancel(action_id, request_options=request_options)
        return response.data


class AsyncActionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawActionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawActionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawActionsClient
        """
        return self._raw_client

    async def create(
        self,
        *,
        idempotency_key: str,
        action: TerminalActionParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateTerminalActionResponse:
        """
        Creates a Terminal action request and sends it to the specified device.

        Parameters
        ----------
        idempotency_key : str
            A unique string that identifies this `CreateAction` request. Keys can be any valid string
            but must be unique for every `CreateAction` request.

            See [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more
            information.

        action : TerminalActionParams
            The Action to create.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateTerminalActionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.terminal.actions.create(
                idempotency_key="thahn-70e75c10-47f7-4ab6-88cc-aaa4076d065e",
                action={
                    "device_id": "{{DEVICE_ID}}",
                    "deadline_duration": "PT5M",
                    "type": "SAVE_CARD",
                    "save_card_options": {
                        "customer_id": "{{CUSTOMER_ID}}",
                        "reference_id": "user-id-1",
                    },
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            idempotency_key=idempotency_key, action=action, request_options=request_options
        )
        return response.data

    async def search(
        self,
        *,
        query: typing.Optional[TerminalActionQueryParams] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchTerminalActionsResponse:
        """
        Retrieves a filtered list of Terminal action requests created by the account making the request. Terminal action requests are available for 30 days.

        Parameters
        ----------
        query : typing.Optional[TerminalActionQueryParams]
            Queries terminal actions based on given conditions and sort order.
            Leaving this unset will return all actions with the default sort order.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for the original query.
            See [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination) for more
            information.

        limit : typing.Optional[int]
            Limit the number of results returned for a single request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchTerminalActionsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.terminal.actions.search(
                query={
                    "filter": {"created_at": {"start_at": "2022-04-01T00:00:00.000Z"}},
                    "sort": {"sort_order": "DESC"},
                },
                limit=2,
            )


        asyncio.run(main())
        """
        response = await self._raw_client.search(
            query=query, cursor=cursor, limit=limit, request_options=request_options
        )
        return response.data

    async def get(
        self, action_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetTerminalActionResponse:
        """
        Retrieves a Terminal action request by `action_id`. Terminal action requests are available for 30 days.

        Parameters
        ----------
        action_id : str
            Unique ID for the desired `TerminalAction`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetTerminalActionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.terminal.actions.get(
                action_id="action_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(action_id, request_options=request_options)
        return response.data

    async def cancel(
        self, action_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CancelTerminalActionResponse:
        """
        Cancels a Terminal action request if the status of the request permits it.

        Parameters
        ----------
        action_id : str
            Unique ID for the desired `TerminalAction`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CancelTerminalActionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.terminal.actions.cancel(
                action_id="action_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.cancel(action_id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class VendorContact(UncheckedBaseModel):
    """
    Represents a contact of a [Vendor](entity:Vendor).
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique Square-generated ID for the [VendorContact](entity:VendorContact).
    This field is required when attempting to update a [VendorContact](entity:VendorContact).
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the [VendorContact](entity:VendorContact).
    This field is required when attempting to create a [Vendor](entity:Vendor).
    """

    email_address: typing.Optional[str] = pydantic.Field(default=None)
    """
    The email address of the [VendorContact](entity:VendorContact).
    """

    phone_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    The phone number of the [VendorContact](entity:VendorContact).
    """

    removed: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The state of the [VendorContact](entity:VendorContact).
    """

    ordinal: int = pydantic.Field()
    """
    The ordinal of the [VendorContact](entity:VendorContact).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .gift_card import GiftCardParams


class GetGiftCardFromNonceResponseParams(typing_extensions.TypedDict):
    """
    A response that contains a `GiftCard` object. If the request resulted in errors,
    the response contains a set of `Error` objects.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    gift_card: typing_extensions.NotRequired[GiftCardParams]
    """
    The retrieved gift card.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .business_hours_period import BusinessHoursPeriodParams


class BusinessHoursParams(typing_extensions.TypedDict):
    """
    The hours of operation for a location.
    """

    periods: typing_extensions.NotRequired[typing.Optional[typing.Sequence[BusinessHoursPeriodParams]]]
    """
    The list of time periods during which the business is open. There can be at most 10 periods per day.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .error import Error
import pydantic
from .payment import Payment
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class GetPaymentResponse(UncheckedBaseModel):
    """
    Defines the response returned by [GetPayment](api-endpoint:Payments-GetPayment).
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Information about errors encountered during the request.
    """

    payment: typing.Optional[Payment] = pydantic.Field(default=None)
    """
    The requested `Payment`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class BookingCustomAttributeDeleteRequest(UncheckedBaseModel):
    """
    Represents an individual delete request in a [BulkDeleteBookingCustomAttributes](api-endpoint:BookingCustomAttributes-BulkDeleteBookingCustomAttributes)
    request. An individual request contains a booking ID, the custom attribute to delete, and an optional idempotency key.
    """

    booking_id: str = pydantic.Field()
    """
    The ID of the target [booking](entity:Booking).
    """

    key: str = pydantic.Field()
    """
    The key of the custom attribute to delete. This key must match the `key` of a
    custom attribute definition in the Square seller account. If the requesting application is not
    the definition owner, you must use the qualified key.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .dispute import DisputeParams


class AcceptDisputeResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields in an `AcceptDispute` response.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information about errors encountered during the request.
    """

    dispute: typing_extensions.NotRequired[DisputeParams]
    """
    Details about the accepted dispute.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .measurement_unit import MeasurementUnitParams
import typing


class StandardUnitDescriptionParams(typing_extensions.TypedDict):
    """
    Contains the name and abbreviation for standard measurement unit.
    """

    unit: typing_extensions.NotRequired[MeasurementUnitParams]
    """
    Identifies the measurement unit being described.
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    UI display name of the measurement unit. For example, 'Pound'.
    """

    abbreviation: typing_extensions.NotRequired[typing.Optional[str]]
    """
    UI display abbreviation for the measurement unit. For example, 'lb'.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .get_customer_response import GetCustomerResponseParams
from .error import ErrorParams


class BulkRetrieveCustomersResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields included in the response body from the
    [BulkRetrieveCustomers](api-endpoint:Customers-BulkRetrieveCustomers) endpoint.
    """

    responses: typing_extensions.NotRequired[typing.Dict[str, GetCustomerResponseParams]]
    """
    A map of responses that correspond to individual retrieve requests, represented by
    key-value pairs.
    
    Each key is the customer ID that was specified for a retrieve request and each value
    is the corresponding response.
    If the request succeeds, the value is the requested customer profile.
    If the request fails, the value contains any errors that occurred during the request.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any top-level errors that prevented the bulk operation from running.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .loyalty_promotion import LoyaltyPromotionParams


class GetLoyaltyPromotionResponseParams(typing_extensions.TypedDict):
    """
    Represents a [RetrieveLoyaltyPromotionPromotions](api-endpoint:Loyalty-RetrieveLoyaltyPromotion) response.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    loyalty_promotion: typing_extensions.NotRequired[LoyaltyPromotionParams]
    """
    The retrieved loyalty promotion.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

CatalogItemFoodAndBeverageDetailsIngredientStandardIngredient = typing.Union[
    typing.Literal[
        "CELERY",
        "CRUSTACEANS",
        "EGGS",
        "FISH",
        "GLUTEN",
        "LUPIN",
        "MILK",
        "MOLLUSCS",
        "MUSTARD",
        "PEANUTS",
        "SESAME",
        "SOY",
        "SULPHITES",
        "TREE_NUTS",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .ach_details import AchDetailsParams
from .error import ErrorParams


class BankAccountPaymentDetailsParams(typing_extensions.TypedDict):
    """
    Additional details about BANK_ACCOUNT type payments.
    """

    bank_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name of the bank associated with the bank account.
    """

    transfer_type: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The type of the bank transfer. The type can be `ACH` or `UNKNOWN`.
    """

    account_ownership_type: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ownership type of the bank account performing the transfer.
    The type can be `INDIVIDUAL`, `COMPANY`, or `ACCOUNT_TYPE_UNKNOWN`.
    """

    fingerprint: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Uniquely identifies the bank account for this seller and can be used
    to determine if payments are from the same bank account.
    """

    country: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The two-letter ISO code representing the country the bank account is located in.
    """

    statement_description: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The statement description as sent to the bank.
    """

    ach_details: typing_extensions.NotRequired[AchDetailsParams]
    """
    ACH-specific information about the transfer. The information is only populated
    if the `transfer_type` is `ACH`.
    """

    errors: typing_extensions.NotRequired[typing.Optional[typing.Sequence[ErrorParams]]]
    """
    Information about errors encountered during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import enum


class SquareEnvironment(enum.Enum):
    PRODUCTION = "https://connect.squareup.com"
    SANDBOX = "https://connect.squareupsandbox.com"



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.invoice_custom_field_placement import InvoiceCustomFieldPlacement


class InvoiceCustomFieldParams(typing_extensions.TypedDict):
    """
    An additional seller-defined and customer-facing field to include on the invoice. For more information,
    see [Custom fields](https://developer.squareup.com/docs/invoices-api/overview#custom-fields).

    Adding custom fields to an invoice requires an
    [Invoices Plus subscription](https://developer.squareup.com/docs/invoices-api/overview#invoices-plus-subscription).
    """

    label: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The label or title of the custom field. This field is required for a custom field.
    """

    value: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The text of the custom field. If omitted, only the label is rendered.
    """

    placement: typing_extensions.NotRequired[InvoiceCustomFieldPlacement]
    """
    The location of the custom field on the invoice. This field is required for a custom field.
    See [InvoiceCustomFieldPlacement](#type-invoicecustomfieldplacement) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
import typing_extensions
import typing_extensions
import typing
from .subscription_phase import SubscriptionPhaseParams
import typing

if typing.TYPE_CHECKING:
    from .catalog_object import CatalogObjectParams


class CatalogSubscriptionPlanParams(typing_extensions.TypedDict):
    """
    Describes a subscription plan. A subscription plan represents what you want to sell in a subscription model, and includes references to each of the associated subscription plan variations.
    For more information, see [Subscription Plans and Variations](https://developer.squareup.com/docs/subscriptions-api/plans-and-variations).
    """

    name: str
    """
    The name of the plan.
    """

    phases: typing_extensions.NotRequired[typing.Optional[typing.Sequence[SubscriptionPhaseParams]]]
    """
    A list of SubscriptionPhase containing the [SubscriptionPhase](entity:SubscriptionPhase) for this plan.
    This field it required. Not including this field will throw a REQUIRED_FIELD_MISSING error
    """

    subscription_plan_variations: typing_extensions.NotRequired[typing.Optional[typing.Sequence["CatalogObjectParams"]]]
    """
    The list of subscription plan variations available for this product
    """

    eligible_item_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    The list of IDs of `CatalogItems` that are eligible for subscription by this SubscriptionPlan's variations.
    """

    eligible_category_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    The list of IDs of `CatalogCategory` that are eligible for subscription by this SubscriptionPlan's variations.
    """

    all_items: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    If true, all items in the merchant's catalog are subscribable by this SubscriptionPlan.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.subscription_event_subscription_event_type import SubscriptionEventSubscriptionEventType
import typing_extensions
from .subscription_event_info import SubscriptionEventInfoParams
import typing
from .phase import PhaseParams


class SubscriptionEventParams(typing_extensions.TypedDict):
    """
    Describes changes to a subscription and the subscription status.
    """

    id: str
    """
    The ID of the subscription event.
    """

    subscription_event_type: SubscriptionEventSubscriptionEventType
    """
    Type of the subscription event.
    See [SubscriptionEventSubscriptionEventType](#type-subscriptioneventsubscriptioneventtype) for possible values
    """

    effective_date: str
    """
    The `YYYY-MM-DD`-formatted date (for example, 2013-01-15) when the subscription event occurred.
    """

    monthly_billing_anchor_date: typing_extensions.NotRequired[int]
    """
    The day-of-the-month the billing anchor date was changed to, if applicable.
    """

    info: typing_extensions.NotRequired[SubscriptionEventInfoParams]
    """
    Additional information about the subscription event.
    """

    phases: typing_extensions.NotRequired[typing.Optional[typing.Sequence[PhaseParams]]]
    """
    A list of Phases, to pass phase-specific information used in the swap.
    """

    plan_variation_id: str
    """
    The ID of the subscription plan variation associated with the subscription.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

CashDrawerEventType = typing.Union[
    typing.Literal[
        "NO_SALE",
        "CASH_TENDER_PAYMENT",
        "OTHER_TENDER_PAYMENT",
        "CASH_TENDER_CANCELLED_PAYMENT",
        "OTHER_TENDER_CANCELLED_PAYMENT",
        "CASH_TENDER_REFUND",
        "OTHER_TENDER_REFUND",
        "PAID_IN",
        "PAID_OUT",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .money import Money
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class OrderLineItemAppliedTax(UncheckedBaseModel):
    """
    Represents an applied portion of a tax to a line item in an order.

    Order-scoped taxes automatically include the applied taxes in each line item.
    Line item taxes must be referenced from any applicable line items.
    The corresponding applied money is automatically computed, based on the
    set of participating line items.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique ID that identifies the applied tax only within this order.
    """

    tax_uid: str = pydantic.Field()
    """
    The `uid` of the tax for which this applied tax represents. It must reference
    a tax present in the `order.taxes` field.
    
    This field is immutable. To change which taxes apply to a line item, delete and add a new
    `OrderLineItemAppliedTax`.
    """

    applied_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The amount of money applied by the tax to the line item.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from ..types.v1order_state import V1OrderState
from .address import AddressParams
from .v1money import V1MoneyParams
from .v1tender import V1TenderParams
from .v1order_history_entry import V1OrderHistoryEntryParams


class V1OrderParams(typing_extensions.TypedDict):
    """
    V1Order
    """

    errors: typing_extensions.NotRequired[typing.Optional[typing.Sequence[ErrorParams]]]
    """
    Any errors that occurred during the request.
    """

    id: typing_extensions.NotRequired[str]
    """
    The order's unique identifier.
    """

    buyer_email: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The email address of the order's buyer.
    """

    recipient_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name of the order's buyer.
    """

    recipient_phone_number: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The phone number to use for the order's delivery.
    """

    state: typing_extensions.NotRequired[V1OrderState]
    """
    Whether the tax is an ADDITIVE tax or an INCLUSIVE tax.
    See [V1OrderState](#type-v1orderstate) for possible values
    """

    shipping_address: typing_extensions.NotRequired[AddressParams]
    """
    The address to ship the order to.
    """

    subtotal_money: typing_extensions.NotRequired[V1MoneyParams]
    """
    The amount of all items purchased in the order, before taxes and shipping.
    """

    total_shipping_money: typing_extensions.NotRequired[V1MoneyParams]
    """
    The shipping cost for the order.
    """

    total_tax_money: typing_extensions.NotRequired[V1MoneyParams]
    """
    The total of all taxes applied to the order.
    """

    total_price_money: typing_extensions.NotRequired[V1MoneyParams]
    """
    The total cost of the order.
    """

    total_discount_money: typing_extensions.NotRequired[V1MoneyParams]
    """
    The total of all discounts applied to the order.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The time when the order was created, in ISO 8601 format.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The time when the order was last modified, in ISO 8601 format.
    """

    expires_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The time when the order expires if no action is taken, in ISO 8601 format.
    """

    payment_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The unique identifier of the payment associated with the order.
    """

    buyer_note: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A note provided by the buyer when the order was created, if any.
    """

    completed_note: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A note provided by the merchant when the order's state was set to COMPLETED, if any
    """

    refunded_note: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A note provided by the merchant when the order's state was set to REFUNDED, if any.
    """

    canceled_note: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A note provided by the merchant when the order's state was set to CANCELED, if any.
    """

    tender: typing_extensions.NotRequired[V1TenderParams]
    """
    The tender used to pay for the order.
    """

    order_history: typing_extensions.NotRequired[typing.Optional[typing.Sequence[V1OrderHistoryEntryParams]]]
    """
    The history of actions associated with the order.
    """

    promo_code: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The promo code provided by the buyer, if any.
    """

    btc_receive_address: typing_extensions.NotRequired[typing.Optional[str]]
    """
    For Bitcoin transactions, the address that the buyer sent Bitcoin to.
    """

    btc_price_satoshi: typing_extensions.NotRequired[typing.Optional[float]]
    """
    For Bitcoin transactions, the price of the buyer's order in satoshi (100 million satoshi equals 1 BTC).
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .cash_drawer_shift import CashDrawerShift
import pydantic
from .error import Error
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class GetCashDrawerShiftResponse(UncheckedBaseModel):
    cash_drawer_shift: typing.Optional[CashDrawerShift] = pydantic.Field(default=None)
    """
    The cash drawer shift queried for.
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Any errors that occurred during the request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .job_assignment import JobAssignment
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class WageSetting(UncheckedBaseModel):
    """
    Represents information about the overtime exemption status, job assignments, and compensation
    for a [team member](entity:TeamMember).
    """

    team_member_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the team member associated with the wage setting.
    """

    job_assignments: typing.Optional[typing.List[JobAssignment]] = pydantic.Field(default=None)
    """
    **Required** The ordered list of jobs that the team member is assigned to.
    The first job assignment is considered the team member's primary job.
    """

    is_overtime_exempt: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the team member is exempt from the overtime rules of the seller's country.
    """

    version: typing.Optional[int] = pydantic.Field(default=None)
    """
    **Read only** Used for resolving concurrency issues. The request fails if the version
    provided does not match the server version at the time of the request. If not provided,
    Square executes a blind write, potentially overwriting data from another write. For more information,
    see [optimistic concurrency](https://developer.squareup.com/docs/working-with-apis/optimistic-concurrency).
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the wage setting was created, in RFC 3339 format.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the wage setting was last updated, in RFC 3339 format.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .location_booking_profile import LocationBookingProfile
import pydantic
from .error import Error
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ListLocationBookingProfilesResponse(UncheckedBaseModel):
    location_booking_profiles: typing.Optional[typing.List[LocationBookingProfile]] = pydantic.Field(default=None)
    """
    The list of a seller's location booking profiles.
    """

    cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    The pagination cursor to be used in the subsequent request to get the next page of the results. Stop retrieving the next page of the results when the cursor is not set.
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Errors that occurred during the request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .shift import ShiftParams
import typing
from .error import ErrorParams


class GetShiftResponseParams(typing_extensions.TypedDict):
    """
    A response to a request to get a `Shift`. The response contains
    the requested `Shift` object and might contain a set of `Error` objects if
    the request resulted in errors.
    """

    shift: typing_extensions.NotRequired[ShiftParams]
    """
    The requested `Shift`.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.job_assignment_pay_type import JobAssignmentPayType
from .money import MoneyParams


class JobAssignmentParams(typing_extensions.TypedDict):
    """
    Represents a job assigned to a [team member](entity:TeamMember), including the compensation the team
    member earns for the job. Job assignments are listed in the team member's [wage setting](entity:WageSetting).
    """

    job_title: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The title of the job.
    """

    pay_type: JobAssignmentPayType
    """
    The current pay type for the job assignment used to
    calculate the pay amount in a pay period.
    See [JobAssignmentPayType](#type-jobassignmentpaytype) for possible values
    """

    hourly_rate: typing_extensions.NotRequired[MoneyParams]
    """
    The hourly pay rate of the job. For `SALARY` pay types, Square calculates the hourly rate based on
    `annual_rate` and `weekly_hours`.
    """

    annual_rate: typing_extensions.NotRequired[MoneyParams]
    """
    The total pay amount for a 12-month period on the job. Set if the job `PayType` is `SALARY`.
    """

    weekly_hours: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The planned hours per week for the job. Set if the job `PayType` is `SALARY`.
    """

    job_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the [job](entity:Job).
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..requests.money import MoneyParams
from ..requests.destination_details_cash_refund_details import DestinationDetailsCashRefundDetailsParams
from ..requests.destination_details_external_refund_details import DestinationDetailsExternalRefundDetailsParams
from ..core.request_options import RequestOptions
from ..core.http_response import HttpResponse
from ..types.refund_payment_response import RefundPaymentResponse
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.get_payment_refund_response import GetPaymentRefundResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.client_wrapper import AsyncClientWrapper
from ..core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawRefundsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def refund_payment(
        self,
        *,
        idempotency_key: str,
        amount_money: MoneyParams,
        app_fee_money: typing.Optional[MoneyParams] = OMIT,
        payment_id: typing.Optional[str] = OMIT,
        destination_id: typing.Optional[str] = OMIT,
        unlinked: typing.Optional[bool] = OMIT,
        location_id: typing.Optional[str] = OMIT,
        customer_id: typing.Optional[str] = OMIT,
        reason: typing.Optional[str] = OMIT,
        payment_version_token: typing.Optional[str] = OMIT,
        team_member_id: typing.Optional[str] = OMIT,
        cash_details: typing.Optional[DestinationDetailsCashRefundDetailsParams] = OMIT,
        external_details: typing.Optional[DestinationDetailsExternalRefundDetailsParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[RefundPaymentResponse]:
        """
        Refunds a payment. You can refund the entire payment amount or a
        portion of it. You can use this endpoint to refund a card payment or record a
        refund of a cash or external payment. For more information, see
        [Refund Payment](https://developer.squareup.com/docs/payments-api/refund-payments).

        Parameters
        ----------
        idempotency_key : str
             A unique string that identifies this `RefundPayment` request. The key can be any valid string
            but must be unique for every `RefundPayment` request.

            Keys are limited to a max of 45 characters - however, the number of allowed characters might be
            less than 45, if multi-byte characters are used.

            For more information, see [Idempotency](https://developer.squareup.com/docs/working-with-apis/idempotency).

        amount_money : MoneyParams
            The amount of money to refund.

            This amount cannot be more than the `total_money` value of the payment minus the total
            amount of all previously completed refunds for this payment.

            This amount must be specified in the smallest denomination of the applicable currency
            (for example, US dollar amounts are specified in cents). For more information, see
            [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts).

            The currency code must match the currency associated with the business
            that is charging the card.

        app_fee_money : typing.Optional[MoneyParams]
            The amount of money the developer contributes to help cover the refunded amount.
            This amount is specified in the smallest denomination of the applicable currency (for example,
            US dollar amounts are specified in cents).

            The value cannot be more than the `amount_money`.

            You can specify this parameter in a refund request only if the same parameter was also included
            when taking the payment. This is part of the application fee scenario the API supports. For more
            information, see [Take Payments and Collect Fees](https://developer.squareup.com/docs/payments-api/take-payments-and-collect-fees).

            To set this field, `PAYMENTS_WRITE_ADDITIONAL_RECIPIENTS` OAuth permission is required.
            For more information, see [Permissions](https://developer.squareup.com/docs/payments-api/take-payments-and-collect-fees#permissions).

        payment_id : typing.Optional[str]
            The unique ID of the payment being refunded.
            Required when unlinked=false, otherwise must not be set.

        destination_id : typing.Optional[str]
            The ID indicating where funds will be refunded to. Required for unlinked refunds. For more
            information, see [Process an Unlinked Refund](https://developer.squareup.com/docs/refunds-api/unlinked-refunds).

            For refunds linked to Square payments, `destination_id` is usually omitted; in this case, funds
            will be returned to the original payment source. The field may be specified in order to request
            a cross-method refund to a gift card. For more information,
            see [Cross-method refunds to gift cards](https://developer.squareup.com/docs/payments-api/refund-payments#cross-method-refunds-to-gift-cards).

        unlinked : typing.Optional[bool]
            Indicates that the refund is not linked to a Square payment.
            If set to true, `destination_id` and `location_id` must be supplied while `payment_id` must not
            be provided.

        location_id : typing.Optional[str]
            The location ID associated with the unlinked refund.
            Required for requests specifying `unlinked=true`.
            Otherwise, if included when `unlinked=false`, will throw an error.

        customer_id : typing.Optional[str]
            The [Customer](entity:Customer) ID of the customer associated with the refund.
            This is required if the `destination_id` refers to a card on file created using the Cards
            API. Only allowed when `unlinked=true`.

        reason : typing.Optional[str]
            A description of the reason for the refund.

        payment_version_token : typing.Optional[str]
             Used for optimistic concurrency. This opaque token identifies the current `Payment`
            version that the caller expects. If the server has a different version of the Payment,
            the update fails and a response with a VERSION_MISMATCH error is returned.
            If the versions match, or the field is not provided, the refund proceeds as normal.

        team_member_id : typing.Optional[str]
            An optional [TeamMember](entity:TeamMember) ID to associate with this refund.

        cash_details : typing.Optional[DestinationDetailsCashRefundDetailsParams]
            Additional details required when recording an unlinked cash refund (`destination_id` is CASH).

        external_details : typing.Optional[DestinationDetailsExternalRefundDetailsParams]
            Additional details required when recording an unlinked external refund
            (`destination_id` is EXTERNAL).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RefundPaymentResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/refunds",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "amount_money": convert_and_respect_annotation_metadata(
                    object_=amount_money, annotation=MoneyParams, direction="write"
                ),
                "app_fee_money": convert_and_respect_annotation_metadata(
                    object_=app_fee_money, annotation=MoneyParams, direction="write"
                ),
                "payment_id": payment_id,
                "destination_id": destination_id,
                "unlinked": unlinked,
                "location_id": location_id,
                "customer_id": customer_id,
                "reason": reason,
                "payment_version_token": payment_version_token,
                "team_member_id": team_member_id,
                "cash_details": convert_and_respect_annotation_metadata(
                    object_=cash_details, annotation=DestinationDetailsCashRefundDetailsParams, direction="write"
                ),
                "external_details": convert_and_respect_annotation_metadata(
                    object_=external_details,
                    annotation=DestinationDetailsExternalRefundDetailsParams,
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RefundPaymentResponse,
                    construct_type(
                        type_=RefundPaymentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, refund_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetPaymentRefundResponse]:
        """
        Retrieves a specific refund using the `refund_id`.

        Parameters
        ----------
        refund_id : str
            The unique ID for the desired `PaymentRefund`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetPaymentRefundResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/refunds/{jsonable_encoder(refund_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetPaymentRefundResponse,
                    construct_type(
                        type_=GetPaymentRefundResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawRefundsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def refund_payment(
        self,
        *,
        idempotency_key: str,
        amount_money: MoneyParams,
        app_fee_money: typing.Optional[MoneyParams] = OMIT,
        payment_id: typing.Optional[str] = OMIT,
        destination_id: typing.Optional[str] = OMIT,
        unlinked: typing.Optional[bool] = OMIT,
        location_id: typing.Optional[str] = OMIT,
        customer_id: typing.Optional[str] = OMIT,
        reason: typing.Optional[str] = OMIT,
        payment_version_token: typing.Optional[str] = OMIT,
        team_member_id: typing.Optional[str] = OMIT,
        cash_details: typing.Optional[DestinationDetailsCashRefundDetailsParams] = OMIT,
        external_details: typing.Optional[DestinationDetailsExternalRefundDetailsParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[RefundPaymentResponse]:
        """
        Refunds a payment. You can refund the entire payment amount or a
        portion of it. You can use this endpoint to refund a card payment or record a
        refund of a cash or external payment. For more information, see
        [Refund Payment](https://developer.squareup.com/docs/payments-api/refund-payments).

        Parameters
        ----------
        idempotency_key : str
             A unique string that identifies this `RefundPayment` request. The key can be any valid string
            but must be unique for every `RefundPayment` request.

            Keys are limited to a max of 45 characters - however, the number of allowed characters might be
            less than 45, if multi-byte characters are used.

            For more information, see [Idempotency](https://developer.squareup.com/docs/working-with-apis/idempotency).

        amount_money : MoneyParams
            The amount of money to refund.

            This amount cannot be more than the `total_money` value of the payment minus the total
            amount of all previously completed refunds for this payment.

            This amount must be specified in the smallest denomination of the applicable currency
            (for example, US dollar amounts are specified in cents). For more information, see
            [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts).

            The currency code must match the currency associated with the business
            that is charging the card.

        app_fee_money : typing.Optional[MoneyParams]
            The amount of money the developer contributes to help cover the refunded amount.
            This amount is specified in the smallest denomination of the applicable currency (for example,
            US dollar amounts are specified in cents).

            The value cannot be more than the `amount_money`.

            You can specify this parameter in a refund request only if the same parameter was also included
            when taking the payment. This is part of the application fee scenario the API supports. For more
            information, see [Take Payments and Collect Fees](https://developer.squareup.com/docs/payments-api/take-payments-and-collect-fees).

            To set this field, `PAYMENTS_WRITE_ADDITIONAL_RECIPIENTS` OAuth permission is required.
            For more information, see [Permissions](https://developer.squareup.com/docs/payments-api/take-payments-and-collect-fees#permissions).

        payment_id : typing.Optional[str]
            The unique ID of the payment being refunded.
            Required when unlinked=false, otherwise must not be set.

        destination_id : typing.Optional[str]
            The ID indicating where funds will be refunded to. Required for unlinked refunds. For more
            information, see [Process an Unlinked Refund](https://developer.squareup.com/docs/refunds-api/unlinked-refunds).

            For refunds linked to Square payments, `destination_id` is usually omitted; in this case, funds
            will be returned to the original payment source. The field may be specified in order to request
            a cross-method refund to a gift card. For more information,
            see [Cross-method refunds to gift cards](https://developer.squareup.com/docs/payments-api/refund-payments#cross-method-refunds-to-gift-cards).

        unlinked : typing.Optional[bool]
            Indicates that the refund is not linked to a Square payment.
            If set to true, `destination_id` and `location_id` must be supplied while `payment_id` must not
            be provided.

        location_id : typing.Optional[str]
            The location ID associated with the unlinked refund.
            Required for requests specifying `unlinked=true`.
            Otherwise, if included when `unlinked=false`, will throw an error.

        customer_id : typing.Optional[str]
            The [Customer](entity:Customer) ID of the customer associated with the refund.
            This is required if the `destination_id` refers to a card on file created using the Cards
            API. Only allowed when `unlinked=true`.

        reason : typing.Optional[str]
            A description of the reason for the refund.

        payment_version_token : typing.Optional[str]
             Used for optimistic concurrency. This opaque token identifies the current `Payment`
            version that the caller expects. If the server has a different version of the Payment,
            the update fails and a response with a VERSION_MISMATCH error is returned.
            If the versions match, or the field is not provided, the refund proceeds as normal.

        team_member_id : typing.Optional[str]
            An optional [TeamMember](entity:TeamMember) ID to associate with this refund.

        cash_details : typing.Optional[DestinationDetailsCashRefundDetailsParams]
            Additional details required when recording an unlinked cash refund (`destination_id` is CASH).

        external_details : typing.Optional[DestinationDetailsExternalRefundDetailsParams]
            Additional details required when recording an unlinked external refund
            (`destination_id` is EXTERNAL).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RefundPaymentResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/refunds",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "amount_money": convert_and_respect_annotation_metadata(
                    object_=amount_money, annotation=MoneyParams, direction="write"
                ),
                "app_fee_money": convert_and_respect_annotation_metadata(
                    object_=app_fee_money, annotation=MoneyParams, direction="write"
                ),
                "payment_id": payment_id,
                "destination_id": destination_id,
                "unlinked": unlinked,
                "location_id": location_id,
                "customer_id": customer_id,
                "reason": reason,
                "payment_version_token": payment_version_token,
                "team_member_id": team_member_id,
                "cash_details": convert_and_respect_annotation_metadata(
                    object_=cash_details, annotation=DestinationDetailsCashRefundDetailsParams, direction="write"
                ),
                "external_details": convert_and_respect_annotation_metadata(
                    object_=external_details,
                    annotation=DestinationDetailsExternalRefundDetailsParams,
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RefundPaymentResponse,
                    construct_type(
                        type_=RefundPaymentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, refund_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetPaymentRefundResponse]:
        """
        Retrieves a specific refund using the `refund_id`.

        Parameters
        ----------
        refund_id : str
            The unique ID for the desired `PaymentRefund`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetPaymentRefundResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/refunds/{jsonable_encoder(refund_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetPaymentRefundResponse,
                    construct_type(
                        type_=GetPaymentRefundResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class AppointmentSegment(UncheckedBaseModel):
    """
    Defines an appointment segment of a booking.
    """

    duration_minutes: typing.Optional[int] = pydantic.Field(default=None)
    """
    The time span in minutes of an appointment segment.
    """

    service_variation_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the [CatalogItemVariation](entity:CatalogItemVariation) object representing the service booked in this segment.
    """

    team_member_id: str = pydantic.Field()
    """
    The ID of the [TeamMember](entity:TeamMember) object representing the team member booked in this segment.
    """

    service_variation_version: typing.Optional[int] = pydantic.Field(default=None)
    """
    The current version of the item variation representing the service booked in this segment.
    """

    intermission_minutes: typing.Optional[int] = pydantic.Field(default=None)
    """
    Time between the end of this segment and the beginning of the subsequent segment.
    """

    any_team_member: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the customer accepts any team member, instead of a specific one, to serve this segment.
    """

    resource_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The IDs of the seller-accessible resources used for this appointment segment.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawObjectClient
from ...requests.catalog_object import CatalogObjectParams
from ...core.request_options import RequestOptions
from ...types.upsert_catalog_object_response import UpsertCatalogObjectResponse
from ...types.get_catalog_object_response import GetCatalogObjectResponse
from ...types.delete_catalog_object_response import DeleteCatalogObjectResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawObjectClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ObjectClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawObjectClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawObjectClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawObjectClient
        """
        return self._raw_client

    def upsert(
        self,
        *,
        idempotency_key: str,
        object: CatalogObjectParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpsertCatalogObjectResponse:
        """
        Creates a new or updates the specified [CatalogObject](entity:CatalogObject).

        To ensure consistency, only one update request is processed at a time per seller account.
        While one (batch or non-batch) update request is being processed, other (batched and non-batched)
        update requests are rejected with the `429` error code.

        Parameters
        ----------
        idempotency_key : str
            A value you specify that uniquely identifies this
            request among all your requests. A common way to create
            a valid idempotency key is to use a Universally unique
            identifier (UUID).

            If you're unsure whether a particular request was successful,
            you can reattempt it with the same idempotency key without
            worrying about creating duplicate objects.

            See [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more information.

        object : CatalogObjectParams
            A CatalogObject to be created or updated.

            - For updates, the object must be active (the `is_deleted` field is not `true`).
            - For creates, the object ID must start with `#`. The provided ID is replaced with a server-generated ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpsertCatalogObjectResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.catalog.object.upsert(
            idempotency_key="af3d1afc-7212-4300-b463-0bfc5314a5ae",
            object={"type": "IMAGE", "id": "#Cocoa"},
        )
        """
        response = self._raw_client.upsert(
            idempotency_key=idempotency_key, object=object, request_options=request_options
        )
        return response.data

    def get(
        self,
        object_id: str,
        *,
        include_related_objects: typing.Optional[bool] = None,
        catalog_version: typing.Optional[int] = None,
        include_category_path_to_root: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetCatalogObjectResponse:
        """
        Returns a single [CatalogItem](entity:CatalogItem) as a
        [CatalogObject](entity:CatalogObject) based on the provided ID. The returned
        object includes all of the relevant [CatalogItem](entity:CatalogItem)
        information including: [CatalogItemVariation](entity:CatalogItemVariation)
        children, references to its
        [CatalogModifierList](entity:CatalogModifierList) objects, and the ids of
        any [CatalogTax](entity:CatalogTax) objects that apply to it.

        Parameters
        ----------
        object_id : str
            The object ID of any type of catalog objects to be retrieved.

        include_related_objects : typing.Optional[bool]
            If `true`, the response will include additional objects that are related to the
            requested objects. Related objects are defined as any objects referenced by ID by the results in the `objects` field
            of the response. These objects are put in the `related_objects` field. Setting this to `true` is
            helpful when the objects are needed for immediate display to a user.
            This process only goes one level deep. Objects referenced by the related objects will not be included. For example,

            if the `objects` field of the response contains a CatalogItem, its associated
            CatalogCategory objects, CatalogTax objects, CatalogImage objects and
            CatalogModifierLists will be returned in the `related_objects` field of the
            response. If the `objects` field of the response contains a CatalogItemVariation,
            its parent CatalogItem will be returned in the `related_objects` field of
            the response.

            Default value: `false`

        catalog_version : typing.Optional[int]
            Requests objects as of a specific version of the catalog. This allows you to retrieve historical
            versions of objects. The value to retrieve a specific version of an object can be found
            in the version field of [CatalogObject](entity:CatalogObject)s. If not included, results will
            be from the current version of the catalog.

        include_category_path_to_root : typing.Optional[bool]
            Specifies whether or not to include the `path_to_root` list for each returned category instance. The `path_to_root` list consists
            of `CategoryPathToRootNode` objects and specifies the path that starts with the immediate parent category of the returned category
            and ends with its root category. If the returned category is a top-level category, the `path_to_root` list is empty and is not returned
            in the response payload.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCatalogObjectResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.catalog.object.get(
            object_id="object_id",
        )
        """
        response = self._raw_client.get(
            object_id,
            include_related_objects=include_related_objects,
            catalog_version=catalog_version,
            include_category_path_to_root=include_category_path_to_root,
            request_options=request_options,
        )
        return response.data

    def delete(
        self, object_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteCatalogObjectResponse:
        """
        Deletes a single [CatalogObject](entity:CatalogObject) based on the
        provided ID and returns the set of successfully deleted IDs in the response.
        Deletion is a cascading event such that all children of the targeted object
        are also deleted. For example, deleting a [CatalogItem](entity:CatalogItem)
        will also delete all of its
        [CatalogItemVariation](entity:CatalogItemVariation) children.

        To ensure consistency, only one delete request is processed at a time per seller account.
        While one (batch or non-batch) delete request is being processed, other (batched and non-batched)
        delete requests are rejected with the `429` error code.

        Parameters
        ----------
        object_id : str
            The ID of the catalog object to be deleted. When an object is deleted, other
            objects in the graph that depend on that object will be deleted as well (for example, deleting a
            catalog item will delete its catalog item variations).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteCatalogObjectResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.catalog.object.delete(
            object_id="object_id",
        )
        """
        response = self._raw_client.delete(object_id, request_options=request_options)
        return response.data


class AsyncObjectClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawObjectClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawObjectClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawObjectClient
        """
        return self._raw_client

    async def upsert(
        self,
        *,
        idempotency_key: str,
        object: CatalogObjectParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpsertCatalogObjectResponse:
        """
        Creates a new or updates the specified [CatalogObject](entity:CatalogObject).

        To ensure consistency, only one update request is processed at a time per seller account.
        While one (batch or non-batch) update request is being processed, other (batched and non-batched)
        update requests are rejected with the `429` error code.

        Parameters
        ----------
        idempotency_key : str
            A value you specify that uniquely identifies this
            request among all your requests. A common way to create
            a valid idempotency key is to use a Universally unique
            identifier (UUID).

            If you're unsure whether a particular request was successful,
            you can reattempt it with the same idempotency key without
            worrying about creating duplicate objects.

            See [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more information.

        object : CatalogObjectParams
            A CatalogObject to be created or updated.

            - For updates, the object must be active (the `is_deleted` field is not `true`).
            - For creates, the object ID must start with `#`. The provided ID is replaced with a server-generated ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpsertCatalogObjectResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.catalog.object.upsert(
                idempotency_key="af3d1afc-7212-4300-b463-0bfc5314a5ae",
                object={"type": "IMAGE", "id": "#Cocoa"},
            )


        asyncio.run(main())
        """
        response = await self._raw_client.upsert(
            idempotency_key=idempotency_key, object=object, request_options=request_options
        )
        return response.data

    async def get(
        self,
        object_id: str,
        *,
        include_related_objects: typing.Optional[bool] = None,
        catalog_version: typing.Optional[int] = None,
        include_category_path_to_root: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetCatalogObjectResponse:
        """
        Returns a single [CatalogItem](entity:CatalogItem) as a
        [CatalogObject](entity:CatalogObject) based on the provided ID. The returned
        object includes all of the relevant [CatalogItem](entity:CatalogItem)
        information including: [CatalogItemVariation](entity:CatalogItemVariation)
        children, references to its
        [CatalogModifierList](entity:CatalogModifierList) objects, and the ids of
        any [CatalogTax](entity:CatalogTax) objects that apply to it.

        Parameters
        ----------
        object_id : str
            The object ID of any type of catalog objects to be retrieved.

        include_related_objects : typing.Optional[bool]
            If `true`, the response will include additional objects that are related to the
            requested objects. Related objects are defined as any objects referenced by ID by the results in the `objects` field
            of the response. These objects are put in the `related_objects` field. Setting this to `true` is
            helpful when the objects are needed for immediate display to a user.
            This process only goes one level deep. Objects referenced by the related objects will not be included. For example,

            if the `objects` field of the response contains a CatalogItem, its associated
            CatalogCategory objects, CatalogTax objects, CatalogImage objects and
            CatalogModifierLists will be returned in the `related_objects` field of the
            response. If the `objects` field of the response contains a CatalogItemVariation,
            its parent CatalogItem will be returned in the `related_objects` field of
            the response.

            Default value: `false`

        catalog_version : typing.Optional[int]
            Requests objects as of a specific version of the catalog. This allows you to retrieve historical
            versions of objects. The value to retrieve a specific version of an object can be found
            in the version field of [CatalogObject](entity:CatalogObject)s. If not included, results will
            be from the current version of the catalog.

        include_category_path_to_root : typing.Optional[bool]
            Specifies whether or not to include the `path_to_root` list for each returned category instance. The `path_to_root` list consists
            of `CategoryPathToRootNode` objects and specifies the path that starts with the immediate parent category of the returned category
            and ends with its root category. If the returned category is a top-level category, the `path_to_root` list is empty and is not returned
            in the response payload.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCatalogObjectResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.catalog.object.get(
                object_id="object_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(
            object_id,
            include_related_objects=include_related_objects,
            catalog_version=catalog_version,
            include_category_path_to_root=include_category_path_to_root,
            request_options=request_options,
        )
        return response.data

    async def delete(
        self, object_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteCatalogObjectResponse:
        """
        Deletes a single [CatalogObject](entity:CatalogObject) based on the
        provided ID and returns the set of successfully deleted IDs in the response.
        Deletion is a cascading event such that all children of the targeted object
        are also deleted. For example, deleting a [CatalogItem](entity:CatalogItem)
        will also delete all of its
        [CatalogItemVariation](entity:CatalogItemVariation) children.

        To ensure consistency, only one delete request is processed at a time per seller account.
        While one (batch or non-batch) delete request is being processed, other (batched and non-batched)
        delete requests are rejected with the `429` error code.

        Parameters
        ----------
        object_id : str
            The ID of the catalog object to be deleted. When an object is deleted, other
            objects in the graph that depend on that object will be deleted as well (for example, deleting a
            catalog item will delete its catalog item variations).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteCatalogObjectResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.catalog.object.delete(
                object_id="object_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.delete(object_id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .error import Error
import pydantic
from .create_vendor_response import CreateVendorResponse
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class BatchCreateVendorsResponse(UncheckedBaseModel):
    """
    Represents an output from a call to [BulkCreateVendors](api-endpoint:Vendors-BulkCreateVendors).
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Any errors that occurred during the request.
    """

    responses: typing.Optional[typing.Dict[str, CreateVendorResponse]] = pydantic.Field(default=None)
    """
    A set of [CreateVendorResponse](entity:CreateVendorResponse) objects encapsulating successfully created [Vendor](entity:Vendor)
    objects or error responses for failed attempts. The set is represented by 
    a collection of idempotency-key/`Vendor`-object or idempotency-key/error-object pairs. The idempotency keys correspond to those specified
    in the input.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .loyalty_program_reward_definition import LoyaltyProgramRewardDefinition
from .catalog_object_reference import CatalogObjectReference
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class LoyaltyProgramRewardTier(UncheckedBaseModel):
    """
    Represents a reward tier in a loyalty program. A reward tier defines how buyers can redeem points for a reward, such as the number of points required and the value and scope of the discount. A loyalty program can offer multiple reward tiers.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Square-assigned ID of the reward tier.
    """

    points: int = pydantic.Field()
    """
    The points exchanged for the reward tier.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the reward tier.
    """

    definition: typing.Optional[LoyaltyProgramRewardDefinition] = pydantic.Field(default=None)
    """
    Provides details about the reward tier definition.
    DEPRECATED at version 2020-12-16. Replaced by the `pricing_rule_reference` field.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the reward tier was created, in RFC 3339 format.
    """

    pricing_rule_reference: CatalogObjectReference = pydantic.Field()
    """
    A reference to the specific version of a `PRICING_RULE` catalog object that contains information about the reward tier discount.
    
    Use `object_id` and `catalog_version` with the [RetrieveCatalogObject](api-endpoint:Catalog-RetrieveCatalogObject) endpoint
    to get discount details. Make sure to set `include_related_objects` to true in the request to retrieve all catalog objects
    that define the discount. For more information, see [Getting discount details for a reward tier](https://developer.squareup.com/docs/loyalty-api/loyalty-rewards#get-discount-details).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .site import SiteParams


class ListSitesResponseParams(typing_extensions.TypedDict):
    """
    Represents a `ListSites` response. The response can include either `sites` or `errors`.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    sites: typing_extensions.NotRequired[typing.Sequence[SiteParams]]
    """
    The sites that belong to the seller.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .webhook_subscription import WebhookSubscriptionParams


class CreateWebhookSubscriptionResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields that are included in the response body of
    a request to the [CreateWebhookSubscription](api-endpoint:WebhookSubscriptions-CreateWebhookSubscription) endpoint.

    Note: if there are errors processing the request, the [Subscription](entity:WebhookSubscription) will not be
    present.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information on errors encountered during the request.
    """

    subscription: typing_extensions.NotRequired[WebhookSubscriptionParams]
    """
    The new [Subscription](entity:WebhookSubscription).
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .error import Error
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CreateMobileAuthorizationCodeResponse(UncheckedBaseModel):
    """
    Defines the fields that are included in the response body of
    a request to the `CreateMobileAuthorizationCode` endpoint.
    """

    authorization_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    The generated authorization code that connects a mobile application instance
    to a Square account.
    """

    expires_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when `authorization_code` expires, in
    [RFC 3339](https://tools.ietf.org/html/rfc3339) format (for example, "2016-09-04T23:59:33.123Z").
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Any errors that occurred during the request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .terminal_refund_query_filter import TerminalRefundQueryFilterParams
from .terminal_refund_query_sort import TerminalRefundQuerySortParams


class TerminalRefundQueryParams(typing_extensions.TypedDict):
    filter: typing_extensions.NotRequired[TerminalRefundQueryFilterParams]
    """
    The filter for the Terminal refund query.
    """

    sort: typing_extensions.NotRequired[TerminalRefundQuerySortParams]
    """
    The sort order for the Terminal refund query.
    """



# This file was auto-generated by Fern from our API Definition.

from ...core.client_wrapper import SyncClientWrapper
import typing
from ...core.request_options import RequestOptions
from ...core.http_response import HttpResponse
from ...types.get_dispute_evidence_response import GetDisputeEvidenceResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...types.delete_dispute_evidence_response import DeleteDisputeEvidenceResponse
from ...core.client_wrapper import AsyncClientWrapper
from ...core.http_response import AsyncHttpResponse


class RawEvidenceClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self, dispute_id: str, evidence_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetDisputeEvidenceResponse]:
        """
        Returns the metadata for the evidence specified in the request URL path.

        You must maintain a copy of any evidence uploaded if you want to reference it later. Evidence cannot be downloaded after you upload it.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute from which you want to retrieve evidence metadata.

        evidence_id : str
            The ID of the evidence to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetDisputeEvidenceResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/disputes/{jsonable_encoder(dispute_id)}/evidence/{jsonable_encoder(evidence_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetDisputeEvidenceResponse,
                    construct_type(
                        type_=GetDisputeEvidenceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self, dispute_id: str, evidence_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeleteDisputeEvidenceResponse]:
        """
        Removes specified evidence from a dispute.
        Square does not send the bank any evidence that is removed.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute from which you want to remove evidence.

        evidence_id : str
            The ID of the evidence you want to remove.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteDisputeEvidenceResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/disputes/{jsonable_encoder(dispute_id)}/evidence/{jsonable_encoder(evidence_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteDisputeEvidenceResponse,
                    construct_type(
                        type_=DeleteDisputeEvidenceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawEvidenceClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self, dispute_id: str, evidence_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetDisputeEvidenceResponse]:
        """
        Returns the metadata for the evidence specified in the request URL path.

        You must maintain a copy of any evidence uploaded if you want to reference it later. Evidence cannot be downloaded after you upload it.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute from which you want to retrieve evidence metadata.

        evidence_id : str
            The ID of the evidence to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetDisputeEvidenceResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/disputes/{jsonable_encoder(dispute_id)}/evidence/{jsonable_encoder(evidence_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetDisputeEvidenceResponse,
                    construct_type(
                        type_=GetDisputeEvidenceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, dispute_id: str, evidence_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeleteDisputeEvidenceResponse]:
        """
        Removes specified evidence from a dispute.
        Square does not send the bank any evidence that is removed.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute from which you want to remove evidence.

        evidence_id : str
            The ID of the evidence you want to remove.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteDisputeEvidenceResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/disputes/{jsonable_encoder(dispute_id)}/evidence/{jsonable_encoder(evidence_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteDisputeEvidenceResponse,
                    construct_type(
                        type_=DeleteDisputeEvidenceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .loyalty_promotion import LoyaltyPromotionParams


class ListLoyaltyPromotionsResponseParams(typing_extensions.TypedDict):
    """
    Represents a [ListLoyaltyPromotions](api-endpoint:Loyalty-ListLoyaltyPromotions) response.
    One of `loyalty_promotions`, an empty object, or `errors` is present in the response.
    If additional results are available, the `cursor` field is also present along with `loyalty_promotions`.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    loyalty_promotions: typing_extensions.NotRequired[typing.Sequence[LoyaltyPromotionParams]]
    """
    The retrieved loyalty promotions.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    The cursor to use in your next call to this endpoint to retrieve the next page of results
    for your original request. This field is present only if the request succeeded and additional
    results are available. For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .time_range import TimeRangeParams
import typing_extensions
import typing
from .segment_filter import SegmentFilterParams


class SearchAvailabilityFilterParams(typing_extensions.TypedDict):
    """
    A query filter to search for buyer-accessible availabilities by.
    """

    start_at_range: TimeRangeParams
    """
    The query expression to search for buy-accessible availabilities with their starting times falling within the specified time range.
    The time range must be at least 24 hours and at most 32 days long.
    For waitlist availabilities, the time range can be 0 or more up to 367 days long.
    """

    location_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The query expression to search for buyer-accessible availabilities with their location IDs matching the specified location ID.
    This query expression cannot be set if `booking_id` is set.
    """

    segment_filters: typing_extensions.NotRequired[typing.Optional[typing.Sequence[SegmentFilterParams]]]
    """
    The query expression to search for buyer-accessible availabilities matching the specified list of segment filters.
    If the size of the `segment_filters` list is `n`, the search returns availabilities with `n` segments per availability.
    
    This query expression cannot be set if `booking_id` is set.
    """

    booking_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The query expression to search for buyer-accessible availabilities for an existing booking by matching the specified `booking_id` value.
    This is commonly used to reschedule an appointment.
    If this expression is set, the `location_id` and `segment_filters` expressions cannot be set.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .search_availability_filter import SearchAvailabilityFilterParams


class SearchAvailabilityQueryParams(typing_extensions.TypedDict):
    """
    The query used to search for buyer-accessible availabilities of bookings.
    """

    filter: SearchAvailabilityFilterParams
    """
    The query filter to search for buyer-accessible availabilities of existing bookings.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

CatalogCustomAttributeDefinitionSellerVisibility = typing.Union[
    typing.Literal["SELLER_VISIBILITY_HIDDEN", "SELLER_VISIBILITY_READ_WRITE_VALUES"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawTerminalClient
from .actions.client import ActionsClient
from .checkouts.client import CheckoutsClient
from .refunds.client import RefundsClient
import typing
from ..core.request_options import RequestOptions
from ..types.dismiss_terminal_action_response import DismissTerminalActionResponse
from ..types.dismiss_terminal_checkout_response import DismissTerminalCheckoutResponse
from ..types.dismiss_terminal_refund_response import DismissTerminalRefundResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawTerminalClient
from .actions.client import AsyncActionsClient
from .checkouts.client import AsyncCheckoutsClient
from .refunds.client import AsyncRefundsClient


class TerminalClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTerminalClient(client_wrapper=client_wrapper)
        self.actions = ActionsClient(client_wrapper=client_wrapper)

        self.checkouts = CheckoutsClient(client_wrapper=client_wrapper)

        self.refunds = RefundsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTerminalClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTerminalClient
        """
        return self._raw_client

    def dismiss_terminal_action(
        self, action_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DismissTerminalActionResponse:
        """
        Dismisses a Terminal action request if the status and type of the request permits it.

        See [Link and Dismiss Actions](https://developer.squareup.com/docs/terminal-api/advanced-features/custom-workflows/link-and-dismiss-actions) for more details.

        Parameters
        ----------
        action_id : str
            Unique ID for the `TerminalAction` associated with the action to be dismissed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DismissTerminalActionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.terminal.dismiss_terminal_action(
            action_id="action_id",
        )
        """
        response = self._raw_client.dismiss_terminal_action(action_id, request_options=request_options)
        return response.data

    def dismiss_terminal_checkout(
        self, checkout_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DismissTerminalCheckoutResponse:
        """
        Dismisses a Terminal checkout request if the status and type of the request permits it.

        Parameters
        ----------
        checkout_id : str
            Unique ID for the `TerminalCheckout` associated with the checkout to be dismissed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DismissTerminalCheckoutResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.terminal.dismiss_terminal_checkout(
            checkout_id="checkout_id",
        )
        """
        response = self._raw_client.dismiss_terminal_checkout(checkout_id, request_options=request_options)
        return response.data

    def dismiss_terminal_refund(
        self, terminal_refund_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DismissTerminalRefundResponse:
        """
        Dismisses a Terminal refund request if the status and type of the request permits it.

        Parameters
        ----------
        terminal_refund_id : str
            Unique ID for the `TerminalRefund` associated with the refund to be dismissed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DismissTerminalRefundResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.terminal.dismiss_terminal_refund(
            terminal_refund_id="terminal_refund_id",
        )
        """
        response = self._raw_client.dismiss_terminal_refund(terminal_refund_id, request_options=request_options)
        return response.data


class AsyncTerminalClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTerminalClient(client_wrapper=client_wrapper)
        self.actions = AsyncActionsClient(client_wrapper=client_wrapper)

        self.checkouts = AsyncCheckoutsClient(client_wrapper=client_wrapper)

        self.refunds = AsyncRefundsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTerminalClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTerminalClient
        """
        return self._raw_client

    async def dismiss_terminal_action(
        self, action_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DismissTerminalActionResponse:
        """
        Dismisses a Terminal action request if the status and type of the request permits it.

        See [Link and Dismiss Actions](https://developer.squareup.com/docs/terminal-api/advanced-features/custom-workflows/link-and-dismiss-actions) for more details.

        Parameters
        ----------
        action_id : str
            Unique ID for the `TerminalAction` associated with the action to be dismissed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DismissTerminalActionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.terminal.dismiss_terminal_action(
                action_id="action_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.dismiss_terminal_action(action_id, request_options=request_options)
        return response.data

    async def dismiss_terminal_checkout(
        self, checkout_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DismissTerminalCheckoutResponse:
        """
        Dismisses a Terminal checkout request if the status and type of the request permits it.

        Parameters
        ----------
        checkout_id : str
            Unique ID for the `TerminalCheckout` associated with the checkout to be dismissed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DismissTerminalCheckoutResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.terminal.dismiss_terminal_checkout(
                checkout_id="checkout_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.dismiss_terminal_checkout(checkout_id, request_options=request_options)
        return response.data

    async def dismiss_terminal_refund(
        self, terminal_refund_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DismissTerminalRefundResponse:
        """
        Dismisses a Terminal refund request if the status and type of the request permits it.

        Parameters
        ----------
        terminal_refund_id : str
            Unique ID for the `TerminalRefund` associated with the refund to be dismissed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DismissTerminalRefundResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.terminal.dismiss_terminal_refund(
                terminal_refund_id="terminal_refund_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.dismiss_terminal_refund(terminal_refund_id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .time_range import TimeRangeParams


class TerminalCheckoutQueryFilterParams(typing_extensions.TypedDict):
    device_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The `TerminalCheckout` objects associated with a specific device. If no device is specified, then all
    `TerminalCheckout` objects for the merchant are displayed.
    """

    created_at: typing_extensions.NotRequired[TimeRangeParams]
    """
    The time range for the beginning of the reporting period, which is inclusive.
    Default value: The current time minus one day.
    Note that `TerminalCheckout`s are available for 30 days after creation.
    """

    status: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Filtered results with the desired status of the `TerminalCheckout`.
    Options: `PENDING`, `IN_PROGRESS`, `CANCEL_REQUESTED`, `CANCELED`, `COMPLETED`
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from .activity_type import ActivityType
from .money import Money
from .payment_balance_activity_app_fee_revenue_detail import PaymentBalanceActivityAppFeeRevenueDetail
from .payment_balance_activity_app_fee_refund_detail import PaymentBalanceActivityAppFeeRefundDetail
from .payment_balance_activity_automatic_savings_detail import PaymentBalanceActivityAutomaticSavingsDetail
from .payment_balance_activity_automatic_savings_reversed_detail import (
    PaymentBalanceActivityAutomaticSavingsReversedDetail,
)
from .payment_balance_activity_charge_detail import PaymentBalanceActivityChargeDetail
from .payment_balance_activity_deposit_fee_detail import PaymentBalanceActivityDepositFeeDetail
from .payment_balance_activity_deposit_fee_reversed_detail import PaymentBalanceActivityDepositFeeReversedDetail
from .payment_balance_activity_dispute_detail import PaymentBalanceActivityDisputeDetail
from .payment_balance_activity_fee_detail import PaymentBalanceActivityFeeDetail
from .payment_balance_activity_free_processing_detail import PaymentBalanceActivityFreeProcessingDetail
from .payment_balance_activity_hold_adjustment_detail import PaymentBalanceActivityHoldAdjustmentDetail
from .payment_balance_activity_open_dispute_detail import PaymentBalanceActivityOpenDisputeDetail
from .payment_balance_activity_other_detail import PaymentBalanceActivityOtherDetail
from .payment_balance_activity_other_adjustment_detail import PaymentBalanceActivityOtherAdjustmentDetail
from .payment_balance_activity_refund_detail import PaymentBalanceActivityRefundDetail
from .payment_balance_activity_release_adjustment_detail import PaymentBalanceActivityReleaseAdjustmentDetail
from .payment_balance_activity_reserve_hold_detail import PaymentBalanceActivityReserveHoldDetail
from .payment_balance_activity_reserve_release_detail import PaymentBalanceActivityReserveReleaseDetail
from .payment_balance_activity_square_capital_payment_detail import PaymentBalanceActivitySquareCapitalPaymentDetail
from .payment_balance_activity_square_capital_reversed_payment_detail import (
    PaymentBalanceActivitySquareCapitalReversedPaymentDetail,
)
from .payment_balance_activity_tax_on_fee_detail import PaymentBalanceActivityTaxOnFeeDetail
from .payment_balance_activity_third_party_fee_detail import PaymentBalanceActivityThirdPartyFeeDetail
from .payment_balance_activity_third_party_fee_refund_detail import PaymentBalanceActivityThirdPartyFeeRefundDetail
from .payment_balance_activity_square_payroll_transfer_detail import PaymentBalanceActivitySquarePayrollTransferDetail
from .payment_balance_activity_square_payroll_transfer_reversed_detail import (
    PaymentBalanceActivitySquarePayrollTransferReversedDetail,
)
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class PayoutEntry(UncheckedBaseModel):
    """
    One or more PayoutEntries that make up a Payout. Each one has a date, amount, and type of activity.
    The total amount of the payout will equal the sum of the payout entries for a batch payout
    """

    id: str = pydantic.Field()
    """
    A unique ID for the payout entry.
    """

    payout_id: str = pydantic.Field()
    """
    The ID of the payout entries’ associated payout.
    """

    effective_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp of when the payout entry affected the balance, in RFC 3339 format.
    """

    type: typing.Optional[ActivityType] = pydantic.Field(default=None)
    """
    The type of activity associated with this payout entry.
    See [ActivityType](#type-activitytype) for possible values
    """

    gross_amount_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The amount of money involved in this payout entry.
    """

    fee_amount_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The amount of Square fees associated with this payout entry.
    """

    net_amount_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The net proceeds from this transaction after any fees.
    """

    type_app_fee_revenue_details: typing.Optional[PaymentBalanceActivityAppFeeRevenueDetail] = pydantic.Field(
        default=None
    )
    """
    Details of any developer app fee revenue generated on a payment.
    """

    type_app_fee_refund_details: typing.Optional[PaymentBalanceActivityAppFeeRefundDetail] = pydantic.Field(
        default=None
    )
    """
    Details of a refund for an app fee on a payment.
    """

    type_automatic_savings_details: typing.Optional[PaymentBalanceActivityAutomaticSavingsDetail] = pydantic.Field(
        default=None
    )
    """
    Details of any automatic transfer from the payment processing balance to the Square Savings account. These are, generally, proportional to the merchant's sales.
    """

    type_automatic_savings_reversed_details: typing.Optional[PaymentBalanceActivityAutomaticSavingsReversedDetail] = (
        pydantic.Field(default=None)
    )
    """
    Details of any automatic transfer from the Square Savings account back to the processing balance. These are, generally, proportional to the merchant's refunds.
    """

    type_charge_details: typing.Optional[PaymentBalanceActivityChargeDetail] = pydantic.Field(default=None)
    """
    Details of credit card payment captures.
    """

    type_deposit_fee_details: typing.Optional[PaymentBalanceActivityDepositFeeDetail] = pydantic.Field(default=None)
    """
    Details of any fees involved with deposits such as for instant deposits.
    """

    type_deposit_fee_reversed_details: typing.Optional[PaymentBalanceActivityDepositFeeReversedDetail] = pydantic.Field(
        default=None
    )
    """
    Details of any reversal or refund of fees involved with deposits such as for instant deposits.
    """

    type_dispute_details: typing.Optional[PaymentBalanceActivityDisputeDetail] = pydantic.Field(default=None)
    """
    Details of any balance change due to a dispute event.
    """

    type_fee_details: typing.Optional[PaymentBalanceActivityFeeDetail] = pydantic.Field(default=None)
    """
    Details of adjustments due to the Square processing fee.
    """

    type_free_processing_details: typing.Optional[PaymentBalanceActivityFreeProcessingDetail] = pydantic.Field(
        default=None
    )
    """
    Square offers Free Payments Processing for a variety of business scenarios including seller referral or when Square wants to apologize for a bug, customer service, repricing complication, and so on. This entry represents details of any credit to the merchant for the purposes of Free Processing.
    """

    type_hold_adjustment_details: typing.Optional[PaymentBalanceActivityHoldAdjustmentDetail] = pydantic.Field(
        default=None
    )
    """
    Details of any adjustment made by Square related to the holding or releasing of a payment.
    """

    type_open_dispute_details: typing.Optional[PaymentBalanceActivityOpenDisputeDetail] = pydantic.Field(default=None)
    """
    Details of any open disputes.
    """

    type_other_details: typing.Optional[PaymentBalanceActivityOtherDetail] = pydantic.Field(default=None)
    """
    Details of any other type that does not belong in the rest of the types.
    """

    type_other_adjustment_details: typing.Optional[PaymentBalanceActivityOtherAdjustmentDetail] = pydantic.Field(
        default=None
    )
    """
    Details of any other type of adjustments that don't fall under existing types.
    """

    type_refund_details: typing.Optional[PaymentBalanceActivityRefundDetail] = pydantic.Field(default=None)
    """
    Details of a refund for an existing card payment.
    """

    type_release_adjustment_details: typing.Optional[PaymentBalanceActivityReleaseAdjustmentDetail] = pydantic.Field(
        default=None
    )
    """
    Details of fees released for adjustments.
    """

    type_reserve_hold_details: typing.Optional[PaymentBalanceActivityReserveHoldDetail] = pydantic.Field(default=None)
    """
    Details of fees paid for funding risk reserve.
    """

    type_reserve_release_details: typing.Optional[PaymentBalanceActivityReserveReleaseDetail] = pydantic.Field(
        default=None
    )
    """
    Details of fees released from risk reserve.
    """

    type_square_capital_payment_details: typing.Optional[PaymentBalanceActivitySquareCapitalPaymentDetail] = (
        pydantic.Field(default=None)
    )
    """
    Details of capital merchant cash advance (MCA) assessments. These are, generally, proportional to the merchant's sales but may be issued for other reasons related to the MCA.
    """

    type_square_capital_reversed_payment_details: typing.Optional[
        PaymentBalanceActivitySquareCapitalReversedPaymentDetail
    ] = pydantic.Field(default=None)
    """
    Details of capital merchant cash advance (MCA) assessment refunds. These are, generally, proportional to the merchant's refunds but may be issued for other reasons related to the MCA.
    """

    type_tax_on_fee_details: typing.Optional[PaymentBalanceActivityTaxOnFeeDetail] = pydantic.Field(default=None)
    """
    Details of tax paid on fee amounts.
    """

    type_third_party_fee_details: typing.Optional[PaymentBalanceActivityThirdPartyFeeDetail] = pydantic.Field(
        default=None
    )
    """
    Details of fees collected by a 3rd party platform.
    """

    type_third_party_fee_refund_details: typing.Optional[PaymentBalanceActivityThirdPartyFeeRefundDetail] = (
        pydantic.Field(default=None)
    )
    """
    Details of refunded fees from a 3rd party platform.
    """

    type_square_payroll_transfer_details: typing.Optional[PaymentBalanceActivitySquarePayrollTransferDetail] = (
        pydantic.Field(default=None)
    )
    """
    Details of a payroll payment that was transferred to a team member’s bank account.
    """

    type_square_payroll_transfer_reversed_details: typing.Optional[
        PaymentBalanceActivitySquarePayrollTransferReversedDetail
    ] = pydantic.Field(default=None)
    """
    Details of a payroll payment to a team member’s bank account that was deposited back to the seller’s account by Square.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .checkout_merchant_settings_payment_methods import CheckoutMerchantSettingsPaymentMethodsParams


class CheckoutMerchantSettingsParams(typing_extensions.TypedDict):
    payment_methods: typing_extensions.NotRequired[CheckoutMerchantSettingsPaymentMethodsParams]
    """
    The set of payment methods accepted for the merchant's account.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the settings were last updated, in RFC 3339 format.
    Examples for January 25th, 2020 6:25:34pm Pacific Standard Time:
    UTC: 2020-01-26T02:25:34Z
    Pacific Standard Time with UTC offset: 2020-01-25T18:25:34-08:00
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .loyalty_event import LoyaltyEventParams


class AccumulateLoyaltyPointsResponseParams(typing_extensions.TypedDict):
    """
    Represents an [AccumulateLoyaltyPoints](api-endpoint:Loyalty-AccumulateLoyaltyPoints) response.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    event: typing_extensions.NotRequired[LoyaltyEventParams]
    """
    The resulting loyalty event. Starting in Square version 2022-08-17, this field is no longer returned.
    """

    events: typing_extensions.NotRequired[typing.Sequence[LoyaltyEventParams]]
    """
    The resulting loyalty events. If the purchase qualifies for points, the `ACCUMULATE_POINTS` event
    is always included. When using the Orders API, the `ACCUMULATE_PROMOTION_POINTS` event is included
    if the purchase also qualifies for a loyalty promotion.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class AfterpayDetails(UncheckedBaseModel):
    """
    Additional details about Afterpay payments.
    """

    email_address: typing.Optional[str] = pydantic.Field(default=None)
    """
    Email address on the buyer's Afterpay account.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .order_line_item_pricing_blocklists_blocked_discount import OrderLineItemPricingBlocklistsBlockedDiscount
import pydantic
from .order_line_item_pricing_blocklists_blocked_tax import OrderLineItemPricingBlocklistsBlockedTax
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class OrderLineItemPricingBlocklists(UncheckedBaseModel):
    """
    Describes pricing adjustments that are blocked from automatic
    application to a line item. For more information, see
    [Apply Taxes and Discounts](https://developer.squareup.com/docs/orders-api/apply-taxes-and-discounts).
    """

    blocked_discounts: typing.Optional[typing.List[OrderLineItemPricingBlocklistsBlockedDiscount]] = pydantic.Field(
        default=None
    )
    """
    A list of discounts blocked from applying to the line item.
    Discounts can be blocked by the `discount_uid` (for ad hoc discounts) or
    the `discount_catalog_object_id` (for catalog discounts).
    """

    blocked_taxes: typing.Optional[typing.List[OrderLineItemPricingBlocklistsBlockedTax]] = pydantic.Field(default=None)
    """
    A list of taxes blocked from applying to the line item.
    Taxes can be blocked by the `tax_uid` (for ad hoc taxes) or
    the `tax_catalog_object_id` (for catalog taxes).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .location import LocationParams


class ListLocationsResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields that are included in the response body of a request
    to the [ListLocations](api-endpoint:Locations-ListLocations) endpoint.

    Either `errors` or `locations` is present in a given response (never both).
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    locations: typing_extensions.NotRequired[typing.Sequence[LocationParams]]
    """
    The business locations.
    """
